using System;
using System.Data;
using System.Collections.Generic;
using System.Text;
using System.Globalization;

using CodePlex.XmlDataMapper;
using CodePlex.XmlDataMapper.Helpers;
using CodePlex.XmlDataMapper.ConsoleClient.Dto;

namespace CodePlex.XmlDataMapper.ConsoleClient.Dal
{
   //Warning: Strongly reccommend you use layering and place DAOs in a seperate DAL layer
   //Keeping folder structure in the demo just for illustration
   public static class OrderDao
   {
      #region simple CRUD

      public static IList<Order> GetAll()
      {
         string storedProcedureName = "OrdersGetAll";
         using (IDataReader reader = BaseDataHelper.ExecuteReader(storedProcedureName, null, false))
         {
            return DTOConverter<Order>.Convert(reader);
         }
      }

      public static Order GetById(int id)
      {
         string storedProcedureName = "OrdersGetById";
         List<GenericDbParameter> paramList = new List<GenericDbParameter>(1);
         paramList.Add(new GenericDbParameter("Id", DbType.Int32, id));
         using (IDataReader reader = BaseDataHelper.ExecuteReader(storedProcedureName, paramList, false))
         {
            IList<Order> list = DTOConverter<Order>.Convert(reader);
            if (list == null | list.Count == 0) return null;
            return list[0];
         }
      }

      public static int Insert(Order order)
      {
         string storedProcedureName = "OrdersInsert";
         List<GenericDbParameter> paramList = new List<GenericDbParameter>(8);
         BaseDataHelper.LoadParamsForDTOInsert(order, paramList);
         paramList.Add(new GenericDbParameter("InsertedId", DbType.Int32, 4, null, ParameterDirection.Output));

         int recordsAffected = BaseDataHelper.ExecuteNonQuery(storedProcedureName, paramList, false);
         int insertedId = (int)paramList[7].Value;
         order.Id = insertedId;
         return insertedId;
      }

      public static int Update(Order order)
      {
         string storedProcedureName = "OrdersUpdate";
         List<GenericDbParameter> paramList = new List<GenericDbParameter>(8);
         BaseDataHelper.LoadParamsForDTOUpdate(order, paramList);
         return BaseDataHelper.ExecuteNonQuery(storedProcedureName, paramList, false);
      }

      public static int Delete(int id)
      {
         string storedProcedureName = "OrdersDelete";
         List<GenericDbParameter> paramList = new List<GenericDbParameter>(1);
         paramList.Add(new GenericDbParameter("Id", DbType.Int32, id));
         return BaseDataHelper.ExecuteNonQuery(storedProcedureName, paramList, false);
      }

      #endregion

      #region Other CRUD 

      //Get Orders by a date range, a sample to demonstrate how custom Get/Retrieve is performed
      public static IList<Order> GetByDateRange(bool isActive, DateTime dateRangeStart, DateTime dateRangeEnd)
      {
         string storedProcedureName = "OrdersGetByDateRange";
         List<GenericDbParameter> paramList = new List<GenericDbParameter>(3);
         paramList.Add(new GenericDbParameter("IsActive", DbType.Boolean, isActive));
         paramList.Add(new GenericDbParameter("DateRangeStart", DbType.DateTime, dateRangeStart));
         paramList.Add(new GenericDbParameter("DateRangeEnd", DbType.DateTime, dateRangeEnd));
         using (IDataReader reader = BaseDataHelper.ExecuteReader(storedProcedureName, paramList, false))
         {
            return DTOConverter<Order>.Convert(reader);
         }
      }

      /// <summary>
      /// Does a batch update for multiple DTOs
      /// </summary>
      /// <param name="list">List or Array of DTOs to be updated</param>
      /// <returns>records updated</returns>
      public static int Update(List<Order> list)
      {
         string storedProcedureName = "OrdersUpdate";
         using (DataSet dataSet = new DataSet())
         {
            dataSet.Locale = CultureInfo.InvariantCulture;
            using (DataTable dataTable = DTOConverter<Order>.ConvertToDataTable(list))
            {
               dataTable.Locale = CultureInfo.InvariantCulture;
               dataSet.Tables.Add(dataTable);
               List<GenericDbParameter> paramList = new List<GenericDbParameter>(8);
               BaseDataHelper.LoadParamsForUpdate(typeof(Order), paramList);
               return BaseDataHelper.UpdateDataSet(storedProcedureName, paramList, dataSet, false);
            }
         }
      }

      public static int Update(Order order, bool withChildren)
      {
         string storedProcedureName1 = "OrdersUpdate";
         string storedProcedureName2 = "OrderDetailsUpdate";
         List<GenericDbParameter> paramList = new List<GenericDbParameter>(8);
         BaseDataHelper.LoadParamsForDTOUpdate(order, paramList);
         int recordsAffected = 0;
         using (BaseDataHelper baseDataHelper = new BaseDataHelper(true))
         {
            recordsAffected = baseDataHelper.ExecuteNonQueryInstance(storedProcedureName1, paramList, false);
            using (DataSet dataSet = DTOConverter<OrderDetail>.ConvertToDataSet(order.OrderDetails))
            {
               List<GenericDbParameter> paramList2 = new List<GenericDbParameter>(4);
               BaseDataHelper.LoadParamsForUpdate(typeof(OrderDetail), paramList2);
               recordsAffected = recordsAffected + baseDataHelper.UpdateDataSetInstance(storedProcedureName2, paramList2, dataSet, false);
            }
            baseDataHelper.Commit();
         }
         return recordsAffected;
      }

      // sample for manually adding parameters and inserting using BaseDataHelper
      public static int InsertManual(Order order)
      {
         string storedProcedureName = "OrdersInsert";
         List<GenericDbParameter> paramList = new List<GenericDbParameter>(8);
         paramList.Add(new GenericDbParameter("Name", DbType.String, 20, order.Name));
         paramList.Add(new GenericDbParameter("OrderDate", DbType.DateTime, 8, order.OrderDate));
         paramList.Add(new GenericDbParameter("IsComplete", DbType.Boolean, 1, order.IsComplete));
         paramList.Add(new GenericDbParameter("Comments", DbType.String, 50, order.Comments));
         paramList.Add(new GenericDbParameter("OrderStatusId", DbType.Byte, 1, order.OrderStatusId));
         paramList.Add(new GenericDbParameter("Amount", DbType.Decimal, 10, order.Amount));
         paramList.Add(new GenericDbParameter("IsActive", DbType.Boolean, 1, order.IsActive));
         paramList.Add(new GenericDbParameter("InsertedId", DbType.Int32, 4, null, ParameterDirection.Output));

         int recordsAffected = BaseDataHelper.ExecuteNonQuery(storedProcedureName, paramList, false);
         int insertedId = (int)paramList[7].Value;
         order.Id = insertedId;
         return insertedId;
      }

      #endregion
   }
}
//Copyright Binoj V Antony  2010 under GNU Library General Public License (LGPL) at http://XmlDataMapper.CodePlex.com/
//Please preserve the authors names and copyright statement in the code even when the work is changed or added to.