using System;
using System.Data;
using System.Configuration;
using System.Collections.Specialized;
using System.Collections;
using System.Globalization;
using System.Reflection;
using System.Collections.Generic;
using System.Text;

using CodePlex.XmlDataMapper.Helpers;

namespace CodePlex.XmlDataMapper
{
   /// <summary>
   /// Static Class that provides helper methods to convert DataTables/DataRows to DTO object/s
   /// Some helper methods to convert DTO objects to DataSets.
   /// Also has helper methods to create table schema in the dataset as per the proposed DTO object.
   /// </summary>
   public static class DTOConverter<T> where T : new()
   {
      private static readonly Type type;
      private static readonly FieldMappers fieldMappers;
      private static readonly int MapperColumnCount;
      private const string ErrorText = "DTOConverterCustomError: problem converting the following: ";

      /*
      public static FieldMappers FieldMappers
      {
          get { return fieldMappers; }
          set
          {
              fieldMappers = value;
              MapperColumnCount = fieldMappers.FieldMapper.Length;
          }
      }
      */

      static DTOConverter()
      {
         type = typeof(T);
         fieldMappers = CacheHelper.GetFromCache(type);
         MapperColumnCount = fieldMappers.FieldMapper.Length;
      }

      public static T CreateObject()
      {
         return new T();
      }

      public static T ConvertForceAllFields(DataRow dataRow)
      {
         T sourceObject = (T)Activator.CreateInstance(type);
         int j = 0;
         try
         {
            for (j = 0; j < MapperColumnCount; j++)
            {
               object fieldValue = dataRow[fieldMappers.FieldMapper[j].ColumnName];
               if (fieldValue == null || fieldValue == DBNull.Value) continue;

               type.GetProperty(fieldMappers.FieldMapper[j].Property).SetValue(sourceObject, fieldValue, null);
            }
         }
         catch (Exception ex)
         {
            throw new Exception(ErrorText + "datarow[\"" + fieldMappers.FieldMapper[j].ColumnName + "\"] to " + type.Name + "." + fieldMappers.FieldMapper[j].Property, ex);
         }
         return sourceObject;
      }

      public static T Convert(DataRow dataRow)
      {
         T sourceObject = (T)Activator.CreateInstance(type);
         FieldMappers strippedFieldMappers = StripMissingColumnsFromMapper(dataRow.Table);
         int strippedMapperColumnCount = strippedFieldMappers.FieldMapper.Length;
         for (int j = 0; j < strippedMapperColumnCount; j++)
         {
            object fieldValue = dataRow[fieldMappers.FieldMapper[j].ColumnName];
            if (fieldValue == null || fieldValue == DBNull.Value)
               continue;
            type.GetProperty(fieldMappers.FieldMapper[j].Property).SetValue(sourceObject, fieldValue, null);
         }
         return sourceObject;
      }

      private static IList<T> ConvertOld(DataTable dataTable)
      {
         if (dataTable == null || dataTable.Rows.Count == 0) return null;
         int rowCount = dataTable.Rows.Count;
         IList<T> list = new List<T>(rowCount);
         FieldMappers fieldMappers = CacheHelper.GetFromCache(type);
         fieldMappers = StripMissingColumnsFromMapper(dataTable);
         int length = fieldMappers.FieldMapper.Length;
         for (int i = 0; i < rowCount; i++)
         {
            T sourceObject = (T)Activator.CreateInstance(type);
            for (int j = 0; j < length; j++)
            {
               object fieldValue = dataTable.Rows[i][fieldMappers.FieldMapper[j].ColumnName];
               if (fieldValue == null || fieldValue == DBNull.Value)
                  continue;
               type.GetProperty(fieldMappers.FieldMapper[j].Property).SetValue(sourceObject, fieldValue, null);
            }
            list.Add(sourceObject);
         }
         return list;
      }

      public static IList<T> Convert(DataTable dataTable)
      {
         if (dataTable == null || dataTable.Rows.Count == 0) return null;
         int rowCount = dataTable.Rows.Count;
         IList<T> list = new List<T>(rowCount);
         FieldMappers strippedFieldMappers = StripMissingColumnsFromMapper(dataTable);
         IList<PropertyInfo> propertyInfoList = GetPropertyInfo(strippedFieldMappers.FieldMapper);
         int[] columnOrdinal = GetOrdinal(strippedFieldMappers.FieldMapper, dataTable);
         int strippedMapperColumnCount = strippedFieldMappers.FieldMapper.Length;

         for (int i = 0; i < rowCount; i++)
         {
            T sourceObject = (T)Activator.CreateInstance(type); 
            for (int j = 0; j < strippedMapperColumnCount; j++) 
            {
               object fieldValue = dataTable.Rows[i][columnOrdinal[j]];
               if (fieldValue == DBNull.Value) continue;

               propertyInfoList[j].SetValue(sourceObject, fieldValue, null);
            }
            list.Add(sourceObject);
         }
         return list;
      }

      public static IList<T> Convert(IDataReader reader)
      {
         if (reader == null || reader.IsClosed || reader.FieldCount == 0) return null;
         DataTable dataTable = reader.GetSchemaTable();
         if (dataTable == null) return null;
         FieldMappers strippedFieldMappers = StripMissingColumnsFromMapperForSchemaTable(dataTable);
         IList<T> list = new List<T>();
         IList<PropertyInfo> propertyInfoList = GetPropertyInfo(strippedFieldMappers.FieldMapper);
         int[] columnOrdinals = GetOrdinalForSchemaTable(strippedFieldMappers.FieldMapper, dataTable);
         int strippedMapperColumnCount = strippedFieldMappers.FieldMapper.Length;

         while (reader.Read())
         {
            T sourceObject = (T)Activator.CreateInstance(type);
            for (int j = 0; j < strippedMapperColumnCount; j++)
            {
               if (reader.IsDBNull(columnOrdinals[j])) continue;
               propertyInfoList[j].SetValue(sourceObject, reader.GetValue(columnOrdinals[j]), null);
            }
            list.Add(sourceObject);
         }
         return list;
      }

      private static IList<PropertyInfo> GetPropertyInfo(FieldMapper[] fieldMapper)
      {
         int length = fieldMapper.Length;
         List<PropertyInfo> list = new List<PropertyInfo>(length);
         for (int i = 0; i < length; i++)
            list.Add(type.GetProperty(fieldMapper[i].Property));
         return list;
      }

      private static IList<PropertyInfo> GetPropertyInfo()
      {
         List<PropertyInfo> list = new List<PropertyInfo>(MapperColumnCount);
         for (int i = 0; i < MapperColumnCount; i++)
            list.Add(type.GetProperty(fieldMappers.FieldMapper[i].Property));
         return list;
      }

      private static FieldMapper[] StripMissingColumnsFromMapper(FieldMapper[] fieldMapper, DataTable dataTable)
      {
         int length = fieldMapper.Length;
         ArrayList arrayList = new ArrayList();
         for (int i = 0; i < length; i++)
            if (dataTable.Columns.Contains(fieldMapper[i].ColumnName) == true)
               arrayList.Add(fieldMapper[i]);

         return (FieldMapper[])arrayList.ToArray(typeof(FieldMapper));
      }

      private static FieldMappers StripMissingColumnsFromMapper(DataTable dataTable)
      {
         ArrayList arrayList = new ArrayList();
         for (int i = 0; i < MapperColumnCount; i++)
            if (dataTable.Columns.Contains(fieldMappers.FieldMapper[i].ColumnName) == true)
               arrayList.Add(fieldMappers.FieldMapper[i]);

         return new FieldMappers((FieldMapper[])arrayList.ToArray(typeof(FieldMapper)));
      }

      private static FieldMappers StripMissingColumnsFromMapperForSchemaTable(DataTable dataTable)
      {
         ArrayList arrayList = new ArrayList();
         //dataTable.PrimaryKey = new DataColumn[1] { dataTable.Columns[0] };
         int rowCount = dataTable.Rows.Count;

         for (int i = 0; i < MapperColumnCount; i++)
            for (int j = 0; j < rowCount; j++)
               if (string.Compare(dataTable.Rows[j][0].ToString(), fieldMappers.FieldMapper[i].ColumnName, StringComparison.OrdinalIgnoreCase) == 0)
               {
                  arrayList.Add(fieldMappers.FieldMapper[i]);
                  break;
               }

         return new FieldMappers((FieldMapper[])arrayList.ToArray(typeof(FieldMapper)));
      }

      private static int[] GetOrdinal(FieldMapper[] fieldMapper, DataTable dataTable)
      {
         int length = fieldMapper.Length;
         ArrayList arrayList = new ArrayList(length);
         for (int i = 0; i < length; i++)
            arrayList.Add(dataTable.Columns.IndexOf(fieldMapper[i].ColumnName));

         return (int[])arrayList.ToArray(typeof(int));
      }

      private static int[] GetOrdinalForSchemaTable(FieldMapper[] fieldMapper, DataTable dataTable)
      {
         int length = fieldMapper.Length;
         ArrayList arrayList = new ArrayList(length);
         int rowCount = dataTable.Rows.Count;
         for (int i = 0; i < length; i++)
            for (int j = 0; j < rowCount; j++)
               if (string.Compare(dataTable.Rows[j][0].ToString(), fieldMapper[i].ColumnName, StringComparison.OrdinalIgnoreCase) == 0)
                  arrayList.Add(j);

         return (int[])arrayList.ToArray(typeof(int));
      }

      private static int[] GetOrdinal(DataTable dataTable)
      {
         ArrayList arrayList = new ArrayList(MapperColumnCount);
         for (int i = 0; i < MapperColumnCount; i++)
            arrayList.Add(dataTable.Columns.IndexOf(fieldMappers.FieldMapper[i].ColumnName));

         return (int[])arrayList.ToArray(typeof(int));
      }

      [Obsolete("Use ConvertToDataSet or ConvertToDataTable instead")]
      public static DataTable Convert(IList<T> list)
      {
         return ConvertToDataTable(list);
      }

      public static DataSet ConvertToDataSet(IList<T> list)
      {
         if (list == null || list.Count == 0) return null;
         DataSet dataSet = new DataSet(type.Name);
         dataSet.Locale = CultureInfo.InvariantCulture;
         DataTable dataTable = ConvertToDataTable(list);
         dataSet.Tables.Add(dataTable);
         return dataSet;
      }

      public static DataTable ConvertToDataTable(IList<T> list)
      {
         if (list == null || list.Count == 0) return null;
         DataTable dataTable = CreateDataTable();
         IList<PropertyInfo> propertyInfoList = GetPropertyInfo();
         int rowCount = list.Count;
         for (int i = 0; i < rowCount; i++)
         {
            DataRow dataRow = dataTable.NewRow();
            for (int j = 0; j < MapperColumnCount; j++)
            {
               object propertyValue = propertyInfoList[j].GetValue(list[i], null);
               if (propertyValue == null || propertyValue == DBNull.Value) continue;

               dataRow[fieldMappers.FieldMapper[j].ColumnName] = propertyValue;
            }
            dataTable.Rows.Add(dataRow);
         }
         return dataTable;
      }

      [Obsolete("Use ConvertToDataSet or ConvertToDataTable instead")]
      public static DataTable Convert(T sourceObject)
      {
         return ConvertToDataTable(sourceObject);
      }

      public static DataSet ConvertToDataSet(T sourceObject)
      {
         if (sourceObject == null) return null;
         DataSet dataSet = new DataSet(type.Name);
         dataSet.Locale = CultureInfo.InvariantCulture;
         DataTable dataTable = ConvertToDataTable(sourceObject);
         dataSet.Tables.Add(dataTable);
         return dataSet;
      }

      public static DataTable ConvertToDataTable(T sourceObject)
      {
         if (sourceObject == null) return null;
         DataTable dataTable = CreateDataTable();
         DataRow dataRow = dataTable.NewRow();
         for (int j = 0; j < MapperColumnCount; j++)
         {
            object propertyValue = type.GetProperty(fieldMappers.FieldMapper[j].Property).GetValue(sourceObject, null);
            if (propertyValue == null || propertyValue == DBNull.Value) continue;

            dataRow[fieldMappers.FieldMapper[j].ColumnName] = propertyValue;
         }
         dataTable.Rows.Add(dataRow);
         return dataTable;
      }

      public static DataSet CreateDataSet()
      {
         DataSet dataSet = new DataSet(type.Name);
         dataSet.Locale = CultureInfo.InvariantCulture;
         DataTable dataTable = CreateDataTable();
         dataSet.Tables.Add(dataTable);
         return dataSet;
      }

      public static DataTable CreateDataTable()
      {
         DataTable dataTable = new DataTable(type.Name);
         dataTable.Locale = CultureInfo.InvariantCulture;
         for (int i = 0; i < MapperColumnCount; i++)
            dataTable.Columns.Add(fieldMappers.FieldMapper[i].ColumnName, fieldMappers.FieldMapper[i].Type);

         return dataTable;
      }

      public static DataTable CreateDataTableWithFullConfiguration()
      {
         DataTable dataTable = new DataTable(type.Name);
         dataTable.Locale = CultureInfo.InvariantCulture;

         for (int i = 0; i < MapperColumnCount; i++)
         {
            FieldMapper fieldMapper = fieldMappers.FieldMapper[i];
            DataColumn dataColumn = new DataColumn(fieldMapper.ColumnName);
            dataColumn.AutoIncrement = fieldMapper.IsIdentity;
            dataColumn.DataType = fieldMapper.Type;
            if (fieldMapper.DefaultValue != null)
               dataColumn.DefaultValue = fieldMapper.DefaultValue;
            if (dataColumn.MaxLength > 0)
               dataColumn.MaxLength = fieldMapper.MaxLength;
            dataTable.Columns.Add(dataColumn);
         }

         return dataTable;
      }
   }
}
//Copyright Binoj V Antony  2010 under GNU Library General Public License (LGPL) at http://XmlDataMapper.CodePlex.com/
//Please preserve the authors names and copyright statement in the code even when the work is changed or added to.