using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Configuration;
using System.Xml;
using System.Xml.Serialization;

using Microsoft.Practices.EnterpriseLibrary.Caching;
using CodePlex.XmlDataMapper.Helpers;

namespace CodePlex.XmlDataMapper.Helpers
{
   /// <summary>
   /// Static Class that provides helper methods to reitrive DTO mapping information.
   /// Also has helper methods to add and remove items from the cache.
   /// </summary>
   public static class CacheHelper
   {
      private static bool isWebApplication;
      private static string rootPath;

      static CacheHelper()
      {
         isWebApplication = HttpContext.Current != null && ConfigurationHelper.IsWebApplication();
         rootPath = ConfigurationHelper.GetRootPath();
      }

      /// <summary>
      /// Retrives the DTO mapping information from the Cache.
      /// Loads the DTO mapping XML as specified in the AppSetting in the Configuration file.
      /// </summary>
      /// <param name="type">Type of the DTO</param>
      /// <returns>
      /// Returns the FieldMappers object that contains an array of the FieldMapper object.
      /// The FieldMapper object contains the mapping between the DTO object property and their
      /// corresponding field in the table.
      /// </returns>
      public static FieldMappers GetFromCache(Type type)
      {
         string typeFullName = type.FullName;
         CacheManager cacheManager = null;
         try
         {
            cacheManager = CacheFactory.GetCacheManager();
         }
         catch (Exception ex)
         {
            throw new ConfigurationErrorsException("EnterpriseLibrary caching configuration not set in the .config file, check faq, if the issue persists then post the problem at http://XmlDataMapper.CodePlex.com/", ex);
         }

         bool success = cacheManager.Contains(typeFullName);
         try
         {
            if (success == false)
               CacheHelper.LoadFileToCache(cacheManager, typeFullName);
         }
         catch (Exception ex)
         {
            throw new ConfigurationErrorsException("Unable to locate/load the datamapping xml configuration file for :" + typeFullName + ", check if the file exists at the required location, if the issue persists then post the problem at http://XmlDataMapper.CodePlex.com/", ex);
         }

         return cacheManager.GetData(typeFullName) as FieldMappers;
      }

      private static void LoadFileToCache(CacheManager cacheManager, string typeFullName)
      {
         XmlSerializer xmlSerializer = new XmlSerializer(typeof(FieldMappers));
         string dtoFilePath = ConfigurationHelper.GetMappingXmlFilePath(typeFullName);
         string configPath = rootPath;
         if (isWebApplication == true)
            configPath = HttpContext.Current.Request.PhysicalApplicationPath;
         configPath = System.IO.Path.Combine(configPath, dtoFilePath);
         //configPath += dtoFilePath;
         XmlReader xmlReader = XmlReader.Create(configPath);
         FieldMappers fieldMappers = new FieldMappers();
         fieldMappers = xmlSerializer.Deserialize(xmlReader) as FieldMappers;
         cacheManager.Add(typeFullName, fieldMappers);
      }

      public static FieldMappers GetFromCache(Type type, bool isWebApplication)
      {
         string typeFullName = type.FullName;
         CacheManager cacheManager = CacheFactory.GetCacheManager();
         bool success = cacheManager.Contains(typeFullName);
         if (success == false)
            CacheHelper.LoadFileToCache(cacheManager, typeFullName, isWebApplication);
         return cacheManager.GetData(typeFullName) as FieldMappers;
      }

      private static void LoadFileToCache(CacheManager cacheManager, string typeFullName, bool isWebApplication)
      {
         XmlSerializer xmlSerializer = new XmlSerializer(typeof(FieldMappers));
         string dtoFilePath = ConfigurationManager.AppSettings[typeFullName];
         string configPath = rootPath;
         if (isWebApplication == true)
            configPath = HttpContext.Current.Request.PhysicalApplicationPath;
         configPath += dtoFilePath;
         XmlReader xmlReader = XmlReader.Create(configPath);
         FieldMappers fieldMappers = new FieldMappers();
         fieldMappers = xmlSerializer.Deserialize(xmlReader) as FieldMappers;
         cacheManager.Add(typeFullName, fieldMappers);
      }

      public static void AddToCache(string key, object value)
      {
         CacheManager cacheManager = CacheFactory.GetCacheManager();
         cacheManager.Add(key, value);
      }

      public static object GetFromCache(string key)
      {
         CacheManager cacheManager = CacheFactory.GetCacheManager();
         return cacheManager.GetData(key);
      }

      public static void RemoveFromCache(string key)
      {
         CacheManager cacheManager = CacheFactory.GetCacheManager();
         cacheManager.Remove(key);
      }

      public static bool CheckIfExistsInCache(string key)
      {
         CacheManager cacheManager = CacheFactory.GetCacheManager();
         return cacheManager.Contains(key);
      }
   }
}
//Copyright Binoj V Antony  2010 under GNU Library General Public License (LGPL) at http://XmlDataMapper.CodePlex.com/
//Please preserve the authors names and copyright statement in the code even when the work is changed or added to.